a.external-link::after {
    /* \00A0 is a non-breaking space
        to make the mark be on the same line as the link
    */
    content: "\00A0[↪]";
}

a.internal-link::after {
    /* \00A0 is a non-breaking space
        to make the mark be on the same line as the link
    */
    content: "\00A0↪";
}
