#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['flashgeotext']

package_data = \
{'': ['*'], 'flashgeotext': ['resources/*', 'resources/.ipynb_checkpoints/*']}

install_requires = \
['flashtext==2.7', 'loguru==0.4.1', 'pydantic==1.4']

extras_require = \
{'doc': ['awscli',
         'mkdocs',
         'mkdocs-material',
         'markdown-include',
         'pydoc-markdown'],
 'test': ['pytest >=4.0.0', 'pytest-cov']}

setup(name='flashgeotext',
      version='0.1.0',
      description='Extract and count countries and cities (+their synonyms) from text',
      author='Benjamin Ramser',
      author_email='ahoi@iwpnd.pw',
      url='https://github.com/iwpnd/flashgeotext',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.6',
     )
